/*
	Animadead: A Skeletal Animation Library
	Copyright (C) 2004 John C. Butterfield

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

	John C. Butterfield
	johnb003@hotmail.com
*/

#include "vertex.h"

#include <iostream>
using namespace std;

Vertex::Vertex()
: x(0), y(0), z(0)
{
}

Vertex::Vertex(float x, float y, float z)
: x(x), y(y), z(z)
{
}

void Vertex::set(float x_arg, float y_arg, float z_arg)
{
	x = x_arg;
	y = y_arg;
	z = z_arg;
}

Vertex &Vertex::operator+=(const Vertex &v)
{
	x += v.x;
	y += v.y;
	z += v.z;
	return *this;
}

Vertex &Vertex::operator-=(const Vertex &v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	return *this;
}

Vertex &Vertex::operator*=(float f)
{
	x *= f;
	y *= f;
	z *= f;
	return *this;
}

Vertex &Vertex::operator/=(float f)
{
	x /= f;
	y /= f;
	z /= f;
	return *this;
}

Vertex operator+(const Vertex &v1, const Vertex &v2)
{
	return Vertex(
		v1.x + v2.x,
		v1.y + v2.y,
		v1.z + v2.z);
}

Vertex operator-(const Vertex &v1, const Vertex &v2)
{
	return Vertex(
		v1.x - v2.x,
		v1.y - v2.y,
		v1.z - v2.z);
}

Vertex operator*(const Vertex &v, const float f)
{
	return Vertex(
		v.x * f,
		v.y * f,
		v.z * f);
}

Vertex operator/(const Vertex &v, const float f)
{
	return Vertex(
		v.x / f,
		v.y / f,
		v.z / f);
}

std::ostream &operator<<(std::ostream &os, const Vertex &v)
{
	os << v.x << "," << v.y << "," << v.z;
	return os;
}

