/* 
   Camera.h

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _CAMERA_H_
#define _CAMERA_H_

#include <iostream>
#include <cmath>
using namespace std;

#include <GL/gl.h>
#include <GL/glu.h>
#include "SDL.h"

#include "FrameRate.h"
#include "Object.h"
#include "Vector3.h"
#include "VectorMath.h"

class Camera
{
public: 
  // Camera interface //
  /** 
   * get an instace of the Camera
   */
  static Camera *getInstance();

  /**
   * Changes the position, view, and upVector of the camera
   */
  void positionCamera (double position[], double view[], double upVector[]);
  void positionCamera (Vector3 position, Vector3 view, Vector3 upVector);
  void positionCamera (double p1, double p2, double p3, double v1, double v2, double v3, double u1, double u2, double u3);
  /**
   * Check for movement based on the keyboard input
   */
  void checkForMovement ();
  /** 
   * rotate the view around a position
   */
  void rotateView (double angle, double x, double y, double z);
  /**
   * move the camera based on the mouse
   */
  void setViewFromMouse ();
  /**
   * moves the camera forward and backward
   */
  void moveCamera (double speed);
  /**
   * strafes the camera left and right
   */
  void strafeCamera (double speed);
  /**
   * walks the camera forward and backward
   */
  void walk(double speed);
  /**
   * Update the camera's data
   */
  void update ();
  /**
   * Call glLookAt
   */
  void look();
  
  void increaseSpeed() { _speed *= 2; }
  void decreaseSpeed() { _speed /= 2; }
  
  void setScreenSize (int width, int height)
  {
    _screenWidth = width;
    _screenHeight = height;
  }
  
  // getter Functions //
  Vector3 getPosition ()
  {
    return _position;
  }
  double getPositionX ()
  {
    return _position.getX ();
  }
  void setPositionX(double t) { _position.setX(t); }
  double getPositionY ()
  {
    return _position.getY ();
  }
  double getPositionZ ()
  {
    return _position.getZ ();
  }
  Vector3 getView ()
  {
    return _view;
  }
  double getViewX ()
  {
    return _view.getX ();
  }
  double getViewY ()
  {
    return _view.getY ();
  }
  double getViewZ ()
  {
    return _view.getZ ();
  }
  Vector3 getUpVector ()
  {
    return _upVector;
  }
  double getUpVectorX ()
  {
    return _upVector.getX ();
  }
  double getUpVectorY ()
  {
    return _upVector.getY ();
  }
  double getUpVectorZ ()
  {
    return _upVector.getZ ();
  }
  Vector3 getStrafe ()
  {
    return _strafe;
  }
  
  void setPositionY(double y) { _position.setY(y); }
  void setViewY(double y) { _view.setY(y); }

  
  double getCurrentFPS() { return _fps; }
  double getSpeed() { return _speed; }

 protected:
  /**
   * Initialize our variables
   */
  void init ();
  
  // Camera variables //
  Vector3 _position; // position of the camera
  Vector3 _view; // view of the camera
  Vector3 _upVector; // up vector
  Vector3 _strafe; // strafe vector
  Vector3 _walk;
  
  double _currentRotationX; // hold the rotation value
  
  double _speed; // the speed at which the camera moves
  double _fps; // Frames Per Second
  
  int _screenWidth;
  int _screenHeight;

 private:
  Camera ();
  virtual ~Camera ();
};

#endif //_CAMERA_H_
