/* 
   IceCreamScoop.cpp

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "IceCreamScoop.h"

IceCreamScoop::IceCreamScoop()
  : Object()
{
  _speed = 0;

  // random color
  double randNum = (double)(rand() % 1000) / 1000.0;
  _color.setX(randNum);
  randNum = (double)(rand() % 1000) / 1000.0;
  _color.setY(randNum);
  randNum = (double)(rand() % 1000) / 1000.0;
  _color.setZ(randNum);

  _hasShadow = true;
  _shadowSize = 0.0;

  loadMesh("mesh/IceCreamScoopSquashed.jbm");
  _squashedMesh = _mesh;
  loadMesh("mesh/IceCreamScoop.jbm");
  // draw the regular scoop
  _drawMesh = _mesh;

  _done = false;
  _squashed = false;
}

IceCreamScoop::~IceCreamScoop()
{
  if (_squashedMesh)
    delete _squashedMesh;
}

/**
 * draw
 */
void IceCreamScoop::draw()
{
  glColor3f(_color.getX(), _color.getY(), _color.getZ());

  glPushMatrix();

  glTranslatef(_currentTranslate.getX() + _initialTranslate.getX(),
	       _currentTranslate.getY() + _initialTranslate.getY(),
	       _currentTranslate.getZ() + _initialTranslate.getZ());
  glRotatef(_currentRotate.getX(), 1, 0, 0);
  glRotatef(_currentRotate.getY(), 0, 1, 0);
  glRotatef(_currentRotate.getZ(), 0, 0, 1);

  if (_drawMesh)
    _drawMesh->draw();

  glPopMatrix();

  if (_hasShadow) // draw a shadow
  {
    double height = _initialTranslate.getY() + _currentTranslate.getY();
    if (height < 1)
      height = 1;

    _shadowSize = 1.0 / height;
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
    glDisable(GL_CULL_FACE);
    glDisable(GL_TEXTURE_2D);
    glPushMatrix();
    glTranslatef(_initialTranslate.getX(), 0.1, _initialTranslate.getZ());

    glScalef(_shadowSize, _shadowSize, _shadowSize);
    glColor4f(0.1, 0.1, 0.1, _shadowSize * 4); // make the shadows show early

    glBegin(GL_TRIANGLES);
    glVertex3f(-0.2, 0.0, -0.2);
    glVertex3f(-0.2, 0.0, 0.2);
    glVertex3f(0.4, 0.0, 0.0);
    glVertex3f(0.2, 0.0, 0.2);
    glVertex3f(0.2, 0.0, -0.2);
    glVertex3f(-0.4, 0.0, 0.0);
    glEnd();
    glPopMatrix();
    glEnable(GL_CULL_FACE);
    glDisable(GL_BLEND);
    glBlendFunc(GL_ONE, GL_ZERO);
  }

  glColor3f(1.0, 1.0, 1.0);
}

/**
 * calculate how far to fall
 */
void IceCreamScoop::fall()
{
  Camera *c = Camera::getInstance();
  
  if (_initialTranslate.getY() + _currentTranslate.getY() < 0.1)
  {
    if (!_squashed)
    {
      _timer.reset();
      _squashed = true;
    }

    _drawMesh = _squashedMesh;
    if (_timer.update() > 1.5)
      _done = true;
  }
  else
  {
    Vector3 v = getTranslate();
    v.setY((double)v.getY() - c->getCurrentFPS());
    setTranslate(v);
  }
}
