/* 
   Object.cpp

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "Object.h"

vector < Object * > Object::_allObjects;
long Object::_currentID = 1;

Object::Object()
{
  _name = "";

  _mesh = 0;
  _ufTexture = 0;
  _collisionRadius = 1.0;

  _id = _currentID;
  Object::addObject(this);
}

Object::~Object()
{
  if (_ufTexture)
    delete _ufTexture;
  if (_mesh)
    delete _mesh;
  
  removeObject(_id);
}

/**
 * draw the mesh
 */
void Object::draw()
{
  glPushMatrix();

  glTranslatef(_currentTranslate.getX() + _initialTranslate.getX(),
	       _currentTranslate.getY() + _initialTranslate.getY(),
	       _currentTranslate.getZ() + _initialTranslate.getZ());
  glRotatef(_currentRotate.getX(), 1, 0, 0);
  glRotatef(_currentRotate.getY(), 0, 1, 0);
  glRotatef(_currentRotate.getZ(), 0, 0, 1);

  if (_mesh)
    _mesh->draw();

  glPopMatrix();
}

/**
 * load a mesh
 */
void Object::loadMesh(string filename)
{
//   ObjectFactory *t = ObjectFactory::getInstance();
//   _mesh = t->loadMesh(filename);

  if (!_ufTexture)
    _ufTexture = new UserFunctionTexture();

  _mesh = new SuperModel(jb::STATIC, _ufTexture);
 
  if (!_mesh)
  {
    cerr << "Error Loading mesh: " << filename << endl;
    return ;
  }

  _mesh->AddMesh(filename);
}
void Object::loadMesh(string filename, vector < string > animationNames, vector < int > animations)
{
  if (!_ufTexture)
    _ufTexture = new UserFunctionTexture();
  
  _mesh = new SuperModel(jb::DEFORMABLE, _ufTexture);
  
  if (!_mesh)
  {
    cerr << "Error Loading mesh: " << filename << endl;
    return ;
  }
  
  // add the animations
  for (int i = 0; i < animationNames.size(); i++)
  {
    unsigned int anim1 = _mesh->AddAnim(animationNames[i]);
    _animations[animations[i]] = anim1;

    // bind to the first one
    if (i == 0)
    {
      for (unsigned int j = 0; j < 49; j++)
	_mesh->Bind(j, anim1); 
    }
  }

  // add the meshes
  _mesh->AddMesh(filename);
}

/**
 * add an animation to the mesh
 */
void Object::addAnimation(string filename, int animation)
{
  if (_mesh)
  {
    unsigned int anim1 = _mesh->AddAnim(filename);
    _animations[animation] = anim1;

    for (unsigned int i = 0; i < 49; i++)
      _mesh->Bind(i, anim1);
  }
}

/**
 * update the time of the animations
 */
void Object::updateAnimation()
{
  if (_mesh)
  {
    Camera *c = Camera::getInstance();
    _mesh->Update(c->getCurrentFPS());
  }
}

/**
 * Change the action of the item
 */
void Object::changeAction(int action)
{
  int an = -1;
  an = _animations[action];
  if (an == -1)
    return;

  for (unsigned int i = 0; i < 49; i++)
    _mesh->Bind(i, (unsigned int)an);
}

/**
 * remove and object from the map
 */
void Object::removeObject(long id)
{

  vector < Object * >::iterator it;
  it = _allObjects.begin();
  for (int i = 0; i < _allObjects.size(); i++)
  {
    if (_allObjects[i]->getID() == id)
    {
      _allObjects.erase(it);
      break;
    }

    it++;
  }
}
