/* 
   Object.h

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _OBJECT_H_
#define _OBJECT_H_

#include "Animations.h"
#include "Camera.h"
#include "ObjectFactory.h"
#include "Vector3.h"
#include "SuperModel.h"

/**
 * Base class for ALL objects
 */
class Object
{
 public:
  Object();
  virtual ~Object();

  /**
   * draw the mesh
   */
  virtual void draw();
 
  /** 
   * getter/setting fuctions
   */
  void setInitialPosition(Vector3 v) { _initialTranslate = v; }
  Vector3 getInitialPosition() { return _initialTranslate; }
  Vector3 getTranslate() { return _currentTranslate; }
  void setTranslate(Vector3 v) { _currentTranslate = v; }
  Vector3 getRotate() { return _currentRotate; }
  void setRotate(Vector3 v) { _currentRotate = v; }

  double getCollisionRadius() { return _collisionRadius; }

  /**
   * update the time of the animations
   */
  virtual void updateAnimation();

  long getID() { return _id; }
  string getName() { return _name; }
  void setName(string s) { _name = s; }

  /**
   * for the object map
   */
  static vector < Object * > getAllObjects() { return _allObjects; }
  static void addObject(Object *t) { _allObjects.push_back(t); _currentID++; }
  static void removeObject(long id);

 protected:
  /**
   * load in a mesh
   */
  virtual void loadMesh(string filename);
  virtual void loadMesh(string filename, vector < string > animationNames, vector < int > animations);
  /**
   * add an animation to the mesh
   */
  virtual void addAnimation(string filename, int animation);
  /**
   * Change the action of the item
   */
  virtual void changeAction(int action);
  map < int, unsigned int > _animations;

  /**
   * Transformations
   */
  Vector3 _initialTranslate;
  Vector3 _currentTranslate;
  Vector3 _initialRotate;
  Vector3 _currentRotate;

  /**
   * the mesh
   */
  SuperModel *_mesh;
  UserFunctionTexture *_ufTexture;

  /**
   * the collision radius for
   *  bounding circle
   */
  double _collisionRadius;

  /**
   * id of the object
   */
  long _id;
  /**
   * name of the object
   */
  string _name;


  /**
   * static map for keeping track
   *  of all the objects
   */
  static vector < Object * > _allObjects;
  static long _currentID;
};

#endif
