/* 
   Texture.cpp

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "Texture.h"

Texture::Texture()
{
  _numTextures = 0;
}

Texture::~Texture()
{
  
}

/**
 * Create a texture from a filename
 */
GLuint Texture::createTexture(string textureName)
{
  // a new surface to hold the texture
  SDL_Surface *textureSurface = 0;
  
  if (_numTextures > 100)
  {
    cerr << __FILE__ << ": " << __LINE__ << ":\t" << "Too Many Textures !! Quitting ...\n";
    return 0;
  }
  
  for (int i = 0; i < _textureNames.size(); i++)
  {
    if (_textureNames[i] == textureName)
      return (GLuint)(_textureList[i]);
  }

  _textureNames.push_back(textureName);

  // load the image file from the string
  textureSurface = IMG_Load(textureName.c_str());
  if (textureSurface)
  {  
    bool alpha = false;
    if ((int)textureSurface->format->Amask != 0)
      alpha = true;

    int num = (alpha) ? 4 : 3;
    int dim = textureSurface->w * textureSurface->h * num;
    GLubyte *data;
    data = (GLubyte *)malloc(sizeof(GLubyte) * dim);
    if (!data)
      cerr << "Error creating data for texture: " << textureName << endl;

    int pos = 0;
    int bytesPerPixel = textureSurface->format->BytesPerPixel;
    unsigned char *pixels = (unsigned char *)(textureSurface->pixels);
    for (int y = textureSurface->h - 1; y > -1; y--)
    {
      for (int x = 0; x < textureSurface->w; x++)
      {
	Uint8 r, g, b, a;
	Uint32 pixelValue = 0;
	
	for (int j = bytesPerPixel - 1; j >= 0; j--)
	{
	  pixelValue = pixelValue << 8; // left shift by 8 bits
	  pixelValue = pixelValue | pixels[(((y * textureSurface->w) + x) * bytesPerPixel) + j];
	}

	if (!alpha)
	  SDL_GetRGB(pixelValue, textureSurface->format, &r, &g, &b);
	else
	  SDL_GetRGBA(pixelValue, textureSurface->format, &r, &g, &b, &a);

	data[pos] = r; pos++;
	data[pos] = g; pos++;
	data[pos] = b; pos++;
	if (alpha)
	{
	  data[pos] = a; pos++;
	}
      }
    }

    int type = (alpha) ? GL_RGBA : GL_RGB;

    // generate a new texture
    glGenTextures(1, _textureList + _numTextures);
    // bind it as the current texture
    glBindTexture(GL_TEXTURE_2D, _textureList[_numTextures]); 

    glTexImage2D(GL_TEXTURE_2D, 0, type, textureSurface->w, textureSurface->h, 0, type, GL_UNSIGNED_BYTE, data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST_MIPMAP_NEAREST);
    gluBuild2DMipmaps(GL_TEXTURE_2D, type, textureSurface->w, textureSurface->h, type, GL_UNSIGNED_BYTE, data);

    if (data)
      free(data);

    /**************    
     // swap this texture into 3d coordinate space
    int width = textureSurface->w; // width of image
    int height = textureSurface->h; // height of image
    unsigned char *data = (unsigned char *)(textureSurface->pixels); // get pixel data
    int bytesPerPixel = textureSurface->format->BytesPerPixel; // bytes per pixel
    cout << textureName << endl;
    
    for (int i = 0; i < (height / 2); i++)
    {
      for (int j = 0; j < width * bytesPerPixel; j+= bytesPerPixel)
      {
	for (int k = 0; k < bytesPerPixel; k++)
	{
	  swap(data[(i * width * bytesPerPixel) + j + k], data[((height - i - 1) * width * bytesPerPixel) + j + k]);
	}
      }
    }
    unsigned char *pixels = new unsigned char [width * height * 3];
    int count = 0;
    
    for (int i = 0; i < (width * height); i++)
    {
      GLbyte r, g, b;
      Uint32 pixelValue = 0;
      
      for (int j = bytesPerPixel - 1; j >= 0; j--)
      {
	pixelValue = pixelValue << 8; // left shift by 8 bits
	pixelValue = pixelValue | data[(i * bytesPerPixel) + j];
      }
      
      SDL_GetRGB(pixelValue, textureSurface->format, (Uint8 *)&r, (Uint8 *)&g, (Uint8 *)&b);
      
      pixels[count++] = r;
      pixels[count++] = g;
      pixels[count++] = b;
      
      pixelValue = 0;
    }
    
    
    // build mimaps 
    cout << "alpha: " << (int)textureSurface->format->alpha << endl;
    cout << "colorkey: " << (int)textureSurface->format->colorkey << endl;
    cout << "amask: " << (int)textureSurface->format->Amask << endl;
    if ((int)textureSurface->format->Amask == 0)
      gluBuild2DMipmaps(GL_TEXTURE_2D, 3, textureSurface->w, textureSurface->h, GL_RGB, GL_UNSIGNED_BYTE, pixels);
    else
      glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textureSurface->w, textureSurface->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, textureSurface->pixels);
      //gluBuild2DMipmaps(GL_TEXTURE_2D, 3, textureSurface->w, textureSurface->h, GL_RGBA, GL_UNSIGNED_BYTE, pixels);
      //gluBuild2DMipmaps(GL_TEXTURE_2D, 3, textureSurface->w, textureSurface->h, GL_RGB, GL_UNSIGNED_BYTE, textureSurface->pixels);

      //glTexImage2D( GL_TEXTURE_2D, 0, 3, textureSurface->w, textureSurface->h, 0, GL_BGR, GL_UNSIGNED_BYTE, pixels);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    ****************************/ 
    
    // free the SDL surface
    SDL_FreeSurface(textureSurface);
  }
  else 
  {
    cerr << __FILE__ << ": " << __LINE__ << ":\t" << "Error Loading texture: " << textureName << ": " << SDL_GetError() << endl;
    //exit(1);
    return 0;
  }
  
  
  // return the texture number, then increment it
  return _textureList[_numTextures++];
}

/**
 * Function to swap texture from 2d to 3d coordinates
 */
void Texture::swap(unsigned char &a, unsigned char &b)
{
  unsigned char temp;
  temp = a;
  a = b;
  b = temp;
}
