/* 
   Vector3.cpp

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "Vector3.h"

Vector3::Vector3()
{
  for (int i = 0; i < 3; i++)
    _points[i] = 0.0;
}

Vector3::Vector3(double x, double y, double z)
{
  _points[0] = x; _points[1] = y; _points[2] = z;
}

Vector3::Vector3(double *points)
{
  for (int i = 0; i < 3; i++)
    _points[i] = points[i];
}

Vector3::~Vector3()
{

}

// overloaded operators
Vector3 Vector3::operator+(Vector3 v1)
{
  return Vector3(v1.getX() + _points[0],
		 v1.getY() + _points[1],
		 v1.getZ() + _points[2]);
}

Vector3 Vector3::operator-(Vector3 v1)
{
  return Vector3(_points[0] - v1.getX(),
		 _points[1] - v1.getY(),
		 _points[2] - v1.getZ());
}

Vector3 Vector3::operator*(double num)
{
  return  Vector3(_points[0] * num,
		  _points[1] * num,
		  _points[2] * num);
}

Vector3 Vector3::operator/(double num)
{
  return Vector3(_points[0] / num,
		 _points[1] / num,
		 _points[2] / num);
}

bool Vector3::operator==(Vector3 v1)
{
  if (v1.getX() == _points[0] && 
      v1.getY() == _points[1] &&
      v1.getZ() == _points[2])
    return true;
  else
    return false;
}

ostream &operator<<(ostream &out, Vector3 &a) 
{
  out << a.getX() << ", " << a.getY() << ", " << a.getZ();
  return out;
}

void Vector3::set(double x, double y, double z)
{
  _points[0] = x;
  _points[1] = y;
  _points[2] = z;
}

void Vector3::set(double *points)
{
  for (int i = 0; i < 3; i++)
    _points[i] = points[i];
}
