/* 
   VectorMath.cpp

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "VectorMath.h"

/**
 * Function to find the cross product 
 *  of two vectors
 */
Vector3 Cross(Vector3 v1, Vector3 v2)
{
	Vector3 normal;
	
	normal.setX((v1.getY() * v2.getZ()) - (v1.getZ() * v2.getY()));
	normal.setY((v1.getZ() * v2.getX()) - (v1.getX() * v2.getZ()));
	normal.setZ((v1.getX() * v2.getY()) - (v1.getY() * v2.getX()));
	
	return normal;
}

/**
 * Normalize a vector
 */
Vector3 Normalize(Vector3 v1)
{
	// get the magnitude
	double magnitude = Magnitude(v1);
	
	// divide the vector by the magnitude
	v1 = v1 / magnitude;
	
	return v1;
}

/**
 * Returns the Magnitude of a vector
 */
double Magnitude(Vector3 v1)
{
	return sqrt((v1.getX() * v1.getX()) +
							(v1.getY() * v1.getY()) +
							(v1.getZ() * v1.getZ()));
}

/**
 * Returns the normal of a triangle
 */
Vector3 FindNormal(Vector3 v1, Vector3 v2, Vector3 v3)
{
  Vector3 f1, f2, cross;

  f1 = v3 - v1;
  f2 = v2 - v1;

  cross = Cross(f1, f2);

  cross = Normalize(cross);

  return cross;
}
