/* 
   World.h

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#ifndef _WORLD_H_
#define _WORLD_H_

#include "Actor.h"
#include "IceCreamScoop.h"
#include "Object.h"
#include "Timer.h"

/**
 * Holds all the objects
 *  in the world
 *    ****
 *  Singleton
 */
class World : public Object
{
 public:
  /**
   * get an instance of the world
   */
  static World *getInstance();

  /**
   * draw the world
   *  and all its objects
   */
  virtual void draw();
  /**
   * update the world
   *  and all its objects
   *
   * returns 
   * -1 if the game is still going on,
   * 0 if draw
   * 1 if player one won
   * 2 if player two won
   */
  virtual int update();

 protected:
  /**
   * vector of all the ice cream scoops
   *  falling
   */
  vector < IceCreamScoop * > _iceCreamScoops;
  /**
   * the two players
   */
  Actor *_player1, *_player2;
  /**
   * time when to create a new scoop
   */
  double _iceCreamScoopSeconds;
  /**
   * the minumum height and ice cream can start at
   */
  int _minIceCreamHeight;
  /**
   * the max height and ice cream can start at
   */
  int _maxIceCreamHeight;
  /**
   * calculate the min and max ice cream height
   */
  void calculateMinMaxIceCreamHeight();
  /**
   * create 3 new scoops at random positions
   */
  void createNewScoops();

  /**
   * check to see if a player
   *  caught an ice cream or
   *  if it hit the floor
   */
  void checkCollision();
  /**
   * check for keyboard/joystick
   *  input for moving the player
   */
  void updatePlayersMovement();

  /**
   * a timer to see when to create new
   *  scoops
   */
  Timer _scoopTimer;
  Timer _gameTimer;

 private:
  World();
  virtual ~World();
};

#endif
