/* 
   main.cpp

   IceCreamDrop 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "main.h"

PFNGLACTIVETEXTUREARBPROC mglActiveTextureARB;
PFNGLMULTITEXCOORD2FARBPROC mglMultiTexCoord2fARB;
PFNGLMULTITEXCOORD1FARBPROC mglMultiTexCoord1fARB;

int main(int argc, char **argv)
{
  // initialize SDL
  if (!initializeSDL())
    return 1;

  // create a new window
  if (!createWindow("IceCreamDrop", 0))
    return 1;

  // check for openGL extensions
  if (isExtensionSupported("GL_ARB_multitexture"))
    cout << "multi texturing supported\n";
  else
    cout << "multi texturing not supported\n";

  // initialize opengl
  initializeGL();

  // initialize opengl extensions
  mglActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC)SDL_GL_GetProcAddress("glActiveTextureARB");
  mglMultiTexCoord2fARB = (PFNGLMULTITEXCOORD2FARBPROC)SDL_GL_GetProcAddress("glMultiTexCoord2fARB");
  mglMultiTexCoord1fARB = (PFNGLMULTITEXCOORD1FARBPROC)SDL_GL_GetProcAddress("glMultiTexCoord1fARB");

  Camera *c = Camera::getInstance();
  c->setScreenSize(1024, 768);
  // give the camera an initial position
  c->positionCamera(Vector3(0.0, 8.0, -13.0), // position
		    Vector3(0.0, 0.0, -2.0), // look at
		    Vector3(0.0, 1.0, 0.0)); // upVector

  // start the render loop
  renderLoop();

  // free the main window
  if (mainWindow)
    SDL_FreeSurface(mainWindow);


  TTF_Quit();
  SDL_Quit();

  // finished !
  return 0;
}


bool initializeSDL()
{
  if (SDL_Init(SDL_INIT_VIDEO) < 0)
  {
    cerr << "Failed initializing SDL Video: " << SDL_GetError() << endl;
    return false;
  }

  // make sure SDL quits
  atexit(SDL_Quit);

  // some video flags
  videoFlags = SDL_OPENGL; // opengl support
  videoFlags |= SDL_HWPALETTE; 

  const SDL_VideoInfo *videoInfo = SDL_GetVideoInfo();
  if (!videoInfo)
  {
    cerr << "Failed getting video info: " << SDL_GetError() << endl;
    return false;
  }

  // check for hardware surfaces
  if (videoInfo->hw_available)
    videoFlags |= SDL_HWSURFACE;
  else
    videoFlags |= SDL_SWSURFACE;
  // enable blitting
  if (videoInfo->blit_hw)
    videoFlags |= SDL_HWACCEL;

  // opengl attributes
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1); // enable double buffering
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, g_depth); // set depth buffer
  SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE, 0);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE, 0);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE, 0);
  SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE, 0);
  SDL_EnableUNICODE(1);

  // initialize SDL_ttf
  if (TTF_Init() < 0)
  {
    cerr << "Error Initializing SDL_ttf: " << SDL_GetError() << endl;
    return false;
  }
  atexit(TTF_Quit);

  return true;
}

bool createWindow(string windowName, bool fullscreen)
{
  // fullscreen mode
  if (fullscreen)
    videoFlags |= SDL_FULLSCREEN;

  mainWindow = SDL_SetVideoMode(g_width, g_height, g_depth, videoFlags);

  if (!mainWindow)
  {
    cerr << "Failed to Create Window: " << SDL_GetError() << endl;
    return false;
  }

  SDL_WM_SetCaption(windowName.c_str(), windowName.c_str());

  return true;
}

bool toggleFullScreen()
{
  if (SDL_WM_ToggleFullScreen(mainWindow) == 0)
  {
    cerr << "Failed to Toggle Full Screen Mode: " << SDL_GetError() << endl;
    return false;
  }

  return true;
}

void initializeGL()
{
  // lights
  GLfloat lightAmbient[]= { 0.5f, 0.5f, 0.5f, 1.0f };
  GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
  GLfloat lightPosition[]= { 0.0f, 0.0f, 1.0f, 1.0f };
  
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_CULL_FACE);
  glColor4f(1.0f, 1.0f, 1.0f, 0.8f);
  glClearColor(0.1, 0.3, 1.0, 0.0);

  glBlendFunc(GL_ONE, GL_ZERO);
  //glBlendFunc(GL_SRC_ALPHA,GL_ONE);
  resizeOpenGL();

  glEnable(GL_NORMALIZE);
  glEnable(GL_COLOR_MATERIAL);
  
  glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);

  //glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0f);
  //glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.05f);
  //glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.01f);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHTING);

  glShadeModel(GL_SMOOTH);
  glEnable(GL_POLYGON_SMOOTH);
  
  //glEnable(GL_LIGHTING);
  //glEnable(GL_LIGHT1); 
}

void resizeOpenGL()
{
  if (g_height == 0)
    g_height = 1; // Don't divide by zero !!

  glViewport(0, 0, g_width, g_height);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  
  gluPerspective(45.0f, (GLfloat)g_width / (GLfloat)g_height, 1.0, 2500.0f);
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void renderLoop()
{
  bool done = false;
  bool isActive = true;
  SDL_Event event; // hold the SDL event
  
  while (!done)
  {
    while (SDL_PollEvent(&event)) // grab the current SDL event
    {
      switch (event.type)
      {
      case SDL_QUIT:
	done = true; // quit
	break;
      case SDL_ACTIVEEVENT:
	/* Something's happend with our focus
	 * If we lost focus or we are iconified, we
	 * shouldn't draw the screen
	 */
	if ( event.active.gain == 0 )
	  isActive = false;
	else
	  isActive = true;
	break;
      case SDL_KEYDOWN:
	switch (event.key.keysym.sym)
	{
	case SDLK_F1: // full screen
	  toggleFullScreen();
	  break;
	case SDLK_ESCAPE: //quit
	  done = 1;
	  break;
	default:
	  break;
	}
	break;
//       case SDL_VIDEORESIZE: // request a resize of the window
// 	int width, height;
// 	width = event.resize.w; height = event.resize.h;
// 	g_width = width;
// 	g_height= height;

// 	Camera *c = Camera::getInstance();
// 	c->setScreenSize(width, height);

// 	mainWindow = SDL_SetVideoMode(event.resize.w, event.resize.h, g_depth, videoFlags);
// 	resizeOpenGL();
// 	if (!mainWindow) // error resizeing
// 	{
// 	  cerr << "Failed resizing Window: " << SDL_GetError() << endl;
// 	  done = 1;
// 	}
// 	break;
      default:
	break;
      }
    }
    
    Camera *camera = Camera::getInstance();
    camera->update();

    World *world = World::getInstance();
    if (isActive)
    {
      int retVal =  world->update();  

      switch (retVal)
      {
      case -1: // continue
	break;
      case 0: // draw
	cout << "draw\n";
	done = 1;
	break;
      case 1: // player1 wins
	cout << "player 1 wins\n";
	done = 1;
	break;
      case 2: // player2 wins
	cout << "player 2 wins\n";
	done = 1;
	break;
      }

      // draw
      renderScene();
    }
  }
}

void renderScene()
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();
 
  Camera *c = Camera::getInstance();
  c->look();

  glColor3f(1.0, 1.0, 1.0);

  World *world = World::getInstance();
  world->draw();
    

  SDL_GL_SwapBuffers();
}

bool isExtensionSupported(string ext)
{
  bool supported = false;
  int pos = 0;

  string extensions ((char *)glGetString(GL_EXTENSIONS));
  cout << "supported extensions: " << extensions << endl;

  pos = string::npos;
  pos = extensions.find(ext);

  if (pos != string::npos)
    return true;

  return false;
}
